@@ -24,13 +24,13 @@ TODO:
 """
 
 __author__ = "Sylvain Thenault"
-__revision__ = "$Id: builder.py,v 1.26 2005/02/11 17:04:21 syt Exp $"
+__revision__ = "$Id: builder.py,v 1.29 2005/04/25 14:47:06 syt Exp $"
 
 import sys
 from compiler import parse
 from inspect import isfunction, ismethod, ismethoddescriptor, isclass, \
-     isbuiltin, getargspec
-from os.path import splitext, basename, dirname, exists
+     isbuiltin, isdatadescriptor, getargspec
+from os.path import splitext, basename, dirname, exists, abspath
 
 from logilab.common.fileutils import norm_read
 from logilab.common.modutils import modpath_from_file
@@ -53,29 +53,43 @@ class ASTNGBuilder:
         self._manager = manager
         self._module = None
         self._done = None
-        self._stack = None
+        self._stack, self._par_stack = None, None
         self._walker = astng.ASTWalker(self)
         
-    def build_from_module(self, module, mod_name=None):
+    def module_build(self, module, modname=None):
         """build an astng from a living module instance
         """
         node = None
-        path = getattr(module, '__file__', None)
         self._module = module
+        path = getattr(module, '__file__', None)
         if path is not None:
             path_, ext = splitext(module.__file__)
             if ext in ('.py', '.pyc', '.pyo') and exists(path_ + '.py'):
-                path = path_ + '.py'
-                node = self.file_build(path)
+                node = self.file_build(path_ + '.py', modname)
         if node is None:
             # this is a built-in module
             # get a partial representation by introspection
             node = self.inspect_build(module)
-            node.file = path
-        node.name = mod_name or module.__name__
-        node.package = hasattr(module, '__path__')
+            node.file = node.path = path and abspath(path) or path
+            node.package = hasattr(module, '__path__')
         return node
 
+    def inspect_build(self, module, modname=None):
+        """build astng from a living module (i.e. using inspect)
+        this is used when there is no python source code available (either
+        because it's a built-in module or because the .py is not available)
+        """
+        self._module = module
+        node = build_module(modname or module.__name__, module.__doc__)
+        node.object = module
+        self._done = {}
+##         if module.__name__ == 'qt':
+##             #print 'hanlding qt !'
+##             #print module.QWidget.__name__, module.QWidget.__module__
+##             self._qt_object_build(node, module)
+##         else:
+        self.object_build(node, module)
+        return node
     
     def file_build(self, path, modname=None):
         """build astng from a source code file (i.e. from an ast)
@@ -87,36 +101,40 @@ class ASTNGBuilder:
         except IOError, ex:
             msg = 'Unable to load file %r (%s)' % (path, ex)
             raise astng.ASTNGBuildingException(msg)
-        # build astng representation
-        try:
-            sys.path.insert(0, dirname(path))
-            node = self.string_build(data)
-        finally:
-            sys.path.pop(0)
-        node.file = node.path = path
+        # get module name if necessary, *before modifying sys.path*
         if modname is None:
             try:
                 modname = '.'.join(modpath_from_file(path))
             except:
                 modname = splitext(basename(path))[0]
-        node.name = modname 
-        node.package = path.find('__init__') > -1
+        # build astng representation
+        try:
+            sys.path.insert(0, dirname(path))
+            node = self.string_build(data, modname, path)
+        finally:
+            sys.path.pop(0)
         return node
     
-    def string_build(self, data):
+    def string_build(self, data, modname='', path=None):
         """build astng from a source code stream (i.e. from an ast)"""
-        return self.ast_build(parse(data + '\n'))
-
-
-    # astng from ast ##########################################################
+        return self.ast_build(parse(data + '\n'), modname, path)
     
-    def ast_build(self, node):
+    def ast_build(self, node, modname=None, path=None):
         """recurse on the ast (soon ng) to add some arguments et method
         """
-        node.pure_python = 1
+        node.file = node.path = abspath(path)
+        if modname.endswith('.__init__'):
+            modname = modname[:-9]
+            node.package = True
+        else:
+            node.package = path and path.find('__init__') > -1 or False
+        node.name = modname 
+        node.pure_python = True
         self._walker.walk(node)
         return node
 
+    # callbacks to build from an existing compiler.ast tree ###################
+
     def visit_module(self, node):
         """visit a stmt.Module node -> init node and push the corresponding
         object or None on the top of the stack
@@ -210,35 +228,48 @@ class ASTNGBuilder:
                 except:
                     #import traceback
                     #traceback.print_exc()
-                    print >> sys.stderr, \
-                          'Unable to get imported names for %r line %s"' % (
-                        node.modname, node.lineno)
+                    continue
+                    # FIXME: log error
+                    #print >> sys.stderr, \
+                    #      'Unable to get imported names for %r line %s"' % (
+                    #    node.modname, node.lineno)
             else:
                 node.parent.set_local(asname or name, node)
+
+    def leave_decorators(self, node):
+        """python >= 2.4
+        visit a stmt.Decorator node -> check for classmethod and staticmethod
+        """
+        func = node.parent
+        for decorator_expr in node.nodes:
+            if isinstance(decorator_expr, astng.Name) and \
+                   decorator_expr.name in ('classmethod', 'staticmethod'):
+                func.type = decorator_expr.name
+        self.leave_default(node)
         
     def visit_assign(self, node):
         """visit a stmt.Assign node -> check for classmethod and staticmethod
         """
         self.visit_default(node)
         klass = node.parent.get_frame()
+        #print node
         if isinstance(klass, astng.Class) and \
             isinstance(node.expr, astng.CallFunc) and \
             isinstance(node.expr.node, astng.Name):
-            name = node.expr.node.name
-            if name in ('classmethod', 'staticmethod'):
+            func_name = node.expr.node.name
+            if func_name in ('classmethod', 'staticmethod'):
                 for ass_node in node.nodes:
                     if isinstance(ass_node, astng.AssName):
                         try:
                             meth = klass.locals[ass_node.name]
                             if isinstance(meth, astng.Function):
-                                meth.type = name
-                            else:
-                                print >> sys.stderr, 'FIXME 1', meth
-                        except Exception:
-                            print >> sys.stderr, 'FIXME 2', ass_node.name
+                                meth.type = func_name
+                            #else:
+                            #    print >> sys.stderr, 'FIXME 1', meth
+                        except KeyError:
+                            #print >> sys.stderr, 'FIXME 2', ass_node.name
                             continue
-                
-            
+
     def visit_assname(self, node):
         """visit a stmt.AssName node -> add name to locals
         """
@@ -246,7 +277,6 @@ class ASTNGBuilder:
         if node.flags == 'OP_ASSIGN':
             node.parent.set_local(node.name, node)
             
-                
     def visit_assattr(self, node):
         """visit a stmt.AssAttr node -> add name to locals, handle members
         definition
@@ -295,23 +325,7 @@ class ASTNGBuilder:
     # this is actually a really minimal representation, including only Module,
     # Function and Class nodes
     
-    def inspect_build(self, module):
-        """build astng from a living module (i.e. using inspect)
-        this is used when there is no python source code available (either
-        because it's a built-in module or because the .py is not available)
-        """
-        node = build_module(module.__name__, module.__doc__)
-        node.object = module
-        self._done = {}
-        if module.__name__ == 'qt':
-            #print 'hanlding qt !'
-            #print module.QWidget.__name__, module.QWidget.__module__
-            self._qt_member_build(node, module)
-        else:
-            self._member_build(node, module)
-        return node
-
-    def _member_build(self, node, obj):
+    def object_build(self, node, obj):
         """recursive method which create a partial ast from real objects
          (only function, class, and method are handled)
         """
@@ -332,14 +346,17 @@ class ASTNGBuilder:
                 # verify this is not an imported function
                 if member.func_code.co_filename != modfile:
                     continue
-                self._function_member_build(node, member)
+                function_object_build(node, member)
             elif ismethoddescriptor(member):
-                self._methoddescriptor_member_build(node, member)
+                methoddescriptor_object_build(node, member)
+            elif isdatadescriptor(member):
+                datadescriptor_object_build(node, member)
+                #node.locals[member.__name__] = astng.Const(None)
             elif isbuiltin(member):
                 # verify this is not an imported member
                 if getattr(member, '__module__', None) != modname:
                     continue
-                self._builtin_member_build(node, member)                
+                builtin_object_build(node, member)                
             elif isclass(member):
                 # verify this is not an imported class
                 #
@@ -347,90 +364,104 @@ class ASTNGBuilder:
                 # __module__ attribute !
                 if getattr(member, '__module__', None) != modname:
                     continue
-                klass = self._class_member_build(node, member)
                 # recursion
-                self._member_build(klass, member)
-
-    def _class_member_build(self, node, member):
-        """create astng for a living class object"""
-        basenames = [base.__name__ for base in member.__bases__]
-        klass = build_class(member.__name__, basenames, member.__doc__)
-        klass.object = member
-        node.add_local_node(klass)
-        return klass
-    
-    def _function_member_build(self, node, member):
-        """create astng for a living function object"""
-        args, varargs, varkw, defaults = getargspec(member)
-        if varargs is not None:
-            args.append(varargs)
-        if varkw is not None:
-            args.append(varkw)
-        func = build_function(member.__name__, args, defaults,
-                              member.func_code.co_flags, member.__doc__)
-        func.object = member
-        node.add_local_node(func)
-    
-    def _methoddescriptor_member_build(self, node, member):
-        """create astng for a living method descriptor object"""
-        # FIXME get arguments ?
-        func = build_function(member.__name__, doc=member.__doc__)
-        # set argnames to None to notice that we have no information, not
-        # and empty argument list
-        func.argnames = None 
-        func.object = member
-        node.add_local_node(func)    
-    _builtin_member_build = _methoddescriptor_member_build
-##     def _builtin_member_build(self, node, member):
-##         """create astng for a living method descriptor object"""
-##         # FIXME get arguments ?
-##         func = build_function(member.__name__, doc=member.__doc__)
-##         func.object = member
-##         node.add_local_node(func)    
+                self.object_build(class_object_build(node, member), member)
     
 
-    def _qt_member_build(self, node, obj):
-        """recursive method which create a partial ast from real objects
-         (only function, class, and method are handled)
-        """
-        if self._done.has_key(obj):
-            return node
-        self._done[obj] = 1
-        # added __main__ to the list to fix problem with qt
-        # (qt.QWidget.__module__ == '__main__') !
-        # and finally also added modutils since __module__ attribute on qt
-        # classes seems to take the name of the importing module
-        # FIXME: bug report on pyqt to remove this special handling...
-        modname = self._module.__name__
-        modules = (modname, '__main__', 'logilab.common.modutils')
-        modfile = getattr(self._module, '__file__', None)
-        for name in dir(obj):
-            try:
-                member = getattr(obj, name)
-            except AttributeError:
-                # damned ExtensionClass.Base, I know you're there !
-                continue
-            if ismethod(member):
-                member = member.im_func
-            if isfunction(member):
-                # verify this is not an imported function
-                if member.func_code.co_filename != modfile:
-                    continue
-                self._function_member_build(node, member)
-            elif ismethoddescriptor(member):
-                self._methoddescriptor_member_build(node, member)
-            elif isbuiltin(member):
-                # verify this is not an imported member
-                if not (member.__module__ is None or member.__module__ in modules):
-                    print 'skipping', member, member.__module__, modules
-                    continue
-                self._builtin_member_build(node, member)
-            elif isclass(member):
-                # verify this is not an imported class
-                if not member.__module__ in modules:
-                    continue
-                # grrr: qt.QWidget.__name__ == 'qt.QWidget' !
-                member.__name__ = member.__name__.split('.')[-1]
-                klass = self._class_member_build(node, member)
-                # recursion
-                self._qt_member_build(klass, member)
+##     def _qt_object_build(self, node, obj):
+##         """recursive method which create a partial ast from real objects
+##          (only function, class, and method are handled)
+##         """
+##         if self._done.has_key(obj):
+##             return node
+##         self._done[obj] = 1
+##         # added __main__ to the list to fix problem with qt
+##         # (qt.QWidget.__module__ == '__main__') !
+##         # and finally also added modutils since __module__ attribute on qt
+##         # classes seems to take the name of the importing module
+##         # FIXME: bug report on pyqt to remove this special handling...
+##         modname = self._module.__name__
+##         modules = (modname, '__main__', 'logilab.common.modutils')
+##         modfile = getattr(self._module, '__file__', None)
+##         for name in dir(obj):
+##             try:
+##                 member = getattr(obj, name)
+##             except AttributeError:
+##                 # damned ExtensionClass.Base, I know you're there !
+##                 continue
+##             if ismethod(member):
+##                 member = member.im_func
+##             if isfunction(member):
+##                 # verify this is not an imported function
+##                 if member.func_code.co_filename != modfile:
+##                     continue
+##                 function_object_build(node, member)
+##             elif ismethoddescriptor(member):
+##                 methoddescriptor_object_build(node, member)
+##             elif isdatadescriptor(member):
+##                 datadescriptor_object_build(node, member)
+##             elif isbuiltin(member):
+##                 # verify this is not an imported member
+##                 if not (member.__module__ is None or member.__module__ in modules):
+##                     #print 'skipping', member, member.__module__, modules
+##                     continue
+##                 builtin_object_build(node, member)
+##             elif isclass(member):
+##                 # verify this is not an imported class
+##                 if not member.__module__ in modules:
+##                     continue
+##                 # grrr: qt.QWidget.__name__ == 'qt.QWidget' !
+##                 member.__name__ = member.__name__.split('.')[-1]
+##                 klass = class_object_build(node, member)
+##                 # recursion
+##                 self._qt_object_build(klass, member)
+
+
+def class_object_build(node, member):
+    """create astng for a living class object"""
+    basenames = [base.__name__ for base in member.__bases__]
+    return _base_class_object_build(node, member, basenames)
+
+def function_object_build(node, member):
+    """create astng for a living function object"""
+    args, varargs, varkw, defaults = getargspec(member)
+    if varargs is not None:
+        args.append(varargs)
+    if varkw is not None:
+        args.append(varkw)
+    func = build_function(member.__name__, args, defaults,
+                          member.func_code.co_flags, member.__doc__)
+    func.object = member
+    node.add_local_node(func)
+
+def datadescriptor_object_build(node, member):
+    """create astng for a living data descriptor object"""
+    return _base_class_object_build(node, member, [])
+
+def methoddescriptor_object_build(node, member):
+    """create astng for a living method descriptor object"""
+    # FIXME get arguments ?
+    func = build_function(member.__name__, doc=member.__doc__)
+    # set argnames to None to notice that we have no information, not
+    # and empty argument list
+    func.argnames = None 
+    func.object = member
+    node.add_local_node(func)
+    
+builtin_object_build = methoddescriptor_object_build
+
+
+def _base_class_object_build(node, member, basenames):
+    """create astng for a living class object, with a given set of base names
+    (e.g. ancestors)
+    """
+    klass = build_class(member.__name__, basenames, member.__doc__)
+    klass.object = member
+    node.add_local_node(klass)
+    return klass
+
+# optimize the tokenize module
+#from logilab.common.bind import optimize_module
+#import tokenize
+#optimize_module(sys.modules['tokenize'], tokenize.__dict__)
+#optimize_module(sys.modules[__name__], sys.modules[__name__].__dict__)
