@@ -19,7 +19,7 @@
 from __future__ import generators
 
 __author__ = "Sylvain Thenault"
-__revision__ = "$Id: utils.py,v 1.11 2005/04/14 09:32:35 syt Exp $"
+__revision__ = "$Id: utils.py,v 1.12 2005/05/02 20:57:28 syt Exp $"
 
 from logilab.common.astng import NotFoundError, manager
 
@@ -199,7 +199,7 @@ def get_interfaces(klass, herited=True, handler_func=iface_hdlr):
         else:
             return
     from logilab.common.astng import Class
-    implements = implements.get_assigned_value()
+    implements = implements.rhs()
     if hasattr(implements, 'nodes'):
         implements = implements.nodes
     else:
