@@ -2,7 +2,7 @@
 
 Some file / path manipulation utilities
 """
-__revision__ = "$Id: unittest_fileutils.py,v 1.16 2005/04/14 09:58:41 syt Exp $"
+__revision__ = "$Id: unittest_fileutils.py,v 1.17 2005/05/02 16:53:46 syt Exp $"
 
 import unittest
 import sys, os, tempfile, shutil
@@ -59,7 +59,7 @@ class GetByExtTC(unittest.TestCase):
         self.assertEquals(files,
                           [join('data', f) for f in ['__init__.py', 'module.py', 'module2.py',
                            'noendingnewline.py', 'nonregr.py', join('sub', 'momo.py')]])
-        files = files_by_ext(DATA_DIR, include_exts=('.py',), exclude_dirs=('sub',))
+        files = files_by_ext(DATA_DIR, include_exts=('.py',), exclude_dirs=('sub', 'CVS'))
         files.sort()
         self.assertEquals(files,
                           [join('data', f) for f in ['__init__.py', 'module.py', 'module2.py',
