@@ -1,7 +1,7 @@
 '''unit tests for ureports.text_writer
 '''
 
-__revision__ = "$Id: unittest_ureports_text.py,v 1.2 2004/05/10 14:48:36 syt Exp $"
+__revision__ = "$Id: unittest_ureports_text.py,v 1.3 2005/04/26 16:51:13 syt Exp $"
 
 import unittest
 from utils import WriterTC
@@ -60,12 +60,12 @@ f22 : v22
 f333: v333
 '''
     advanced_table = '''
-field                value 
-:::::::::::::::::::::::::::
-f1                   v1    
-f22                  v22   
-f333                 v333  
-http://www.perdu.com 
+field          value 
+:::::::::::::::::::::
+f1             v1    
+f22            v22   
+f333           v333  
+`toi perdu ?`_ 
 '''
 
 
